/*
 * A neat little GAIM plugin to integrate with GNU Talk Filters.
 * I started with the 'spellchk' code and modified it accordingly.
 *
 * To build: Drop into "plugins" directory in GAIM source tree. Set
 * PLUGIN_LIBS environment variable to "-ltalkfilters", and type
 * "make talkfilters.so". Then drop talkfilters.so into $prefix/lib/gaim and
 * restart GAIM.
 *
 * Mark Lindner <markl@gnu.org> 1/6/04
 */

#include "internal.h"
#include "gtkgaim.h"
#include "version.h"

#include "debug.h"
#include "signals.h"
#include "util.h"

#include "gtkplugin.h"
#include "gtkutils.h"

#include <stdio.h>
#include <string.h>
#ifndef _WIN32
#include <strings.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>

#include <talkfilters.h>

#define TALKFILTERS_PLUGIN_ID "gtk-talkfilters"
#define TALKFILTERS_VERSION "0.1"
#define TALKFILTERS_WEBSITE "http://www.dystance.net/software/talkfilters"

static gtf_filter_t *current_filter = NULL;

/*
 */

static gboolean translate_message(GaimAccount *account, GaimConversation *conv,
                                  char **message, void *data)
{
   if (message == NULL || *message == NULL)
      return FALSE;

   if(current_filter != NULL)
   {
      char *tmp;
   
      size_t len = strlen(*message);
      if(len < 40)
         len += 40;
      else
         len *= 2;
      
      tmp = (char *)g_malloc(len);

      current_filter->filter(*message, tmp, len);
      g_free(*message);
      *message = tmp;
   }
   
   return FALSE;
}

/*
 */

static void select_filter(GtkObject *m, void *arg)
{
   current_filter = (gtf_filter_t *)gtk_object_get_user_data(m);
}

/*
 *  EXPORTED FUNCTIONS
 */

static gboolean plugin_load(GaimPlugin *plugin)
{
   void *conv_handle = gaim_conversations_get_handle();

   gaim_signal_connect(conv_handle, "writing-im-msg",
                       plugin, GAIM_CALLBACK(translate_message), NULL);
   gaim_signal_connect(conv_handle, "writing-chat-msg",
                       plugin, GAIM_CALLBACK(translate_message), NULL);

   return TRUE;
}

/*
 */

static GtkWidget *get_config_frame(GaimPlugin *plugin)
{
   GtkWidget *ret, *vbox, *hbox, *label, *menu, *opt, *optmenu;
   const gtf_filter_t *filter;
   int ct;

   // create filter list
   
   menu = gtk_menu_new();

   opt = gtk_menu_item_new_with_label("(None)");
   gtk_object_set_user_data(GTK_OBJECT(opt), NULL);
   gtk_signal_connect(GTK_OBJECT(opt), "activate",
                      GTK_SIGNAL_FUNC(select_filter), NULL);
   gtk_widget_show(opt);
   gtk_menu_append(GTK_MENU(menu), opt);

   ct = gtf_filter_count();
   for(filter = gtf_filter_list(); ct; filter++, ct--)
   {
      opt = gtk_menu_item_new_with_label(filter->desc);
      gtk_signal_connect(GTK_OBJECT(opt), "activate",
                         GTK_SIGNAL_FUNC(select_filter), NULL);
      gtk_object_set_user_data(GTK_OBJECT(opt), (void *)filter);
      gtk_widget_show(opt);
      gtk_menu_append(GTK_MENU(menu), opt);
   }

   // create config UI
   
   ret = gtk_vbox_new(FALSE, 18);
   gtk_container_set_border_width (GTK_CONTAINER (ret), 12);

   vbox = gaim_gtk_make_frame(ret, _("Talk Filters"));
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
   gtk_widget_set_size_request(vbox, 300, -1);
   gtk_widget_show (vbox);

   hbox = gtk_hbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(vbox), hbox);
   
   label = gtk_label_new(_("Active filter: "));
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);

   optmenu = gtk_option_menu_new();

   gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), menu);
   gtk_option_menu_set_history(GTK_OPTION_MENU(optmenu), 0);
   
   gtk_box_pack_start(GTK_BOX(hbox), optmenu, FALSE, FALSE, 5);
      
   gtk_widget_show_all(ret);
   return ret;
}

/*
 */

static GaimGtkPluginUiInfo ui_info =
{
   get_config_frame
};

/*
 */

static GaimPluginInfo info =
{
   GAIM_PLUGIN_MAGIC,
   GAIM_MAJOR_VERSION,
   GAIM_MINOR_VERSION,
   GAIM_PLUGIN_STANDARD,
   GAIM_GTK_PLUGIN_TYPE,
   0,
   NULL,
   GAIM_PRIORITY_DEFAULT,
   TALKFILTERS_PLUGIN_ID,
   N_("GNU Talk Filters"),
   TALKFILTERS_VERSION,
   N_("Translates text in outgoing messages into humorous dialects."),
   N_("Translates text in outgoing messages into humorous dialects."),
   "Mark Lindner <markl@gnu.org>",
   TALKFILTERS_WEBSITE,
   plugin_load,
   NULL,
   NULL,
   &ui_info,
   NULL
};

/*
 */

static void init_plugin(GaimPlugin *plugin)
{
}

GAIM_INIT_PLUGIN(talkfilters, init_plugin, info)

// eof
